# *************************************************************************
#
# Copyright (c) 2021 Andrei Gramakov. All rights reserved.
#
# This file is licensed under the terms of the MIT license.
# For a copy, see: https://opensource.org/licenses/MIT
#
# site:    https://agramakov.me
# e-mail:  mail@agramakov.me
#
# *************************************************************************

import sys
import os


def wait_key() -> int:
    '''
    Wait for a key press on the console and return it.

    Returns keycode as integer
    '''
    result = None
    if os.name == 'nt':
        import msvcrt
        result = msvcrt.getch()
    else:
        import termios
        fd = sys.stdin.fileno()

        oldterm = termios.tcgetattr(fd)
        newattr = termios.tcgetattr(fd)
        newattr[3] = newattr[3] & ~termios.ICANON & ~termios.ECHO
        termios.tcsetattr(fd, termios.TCSANOW, newattr)

        try:
            result = sys.stdin.read(1)
        except IOError:
            pass
        finally:
            termios.tcsetattr(fd, termios.TCSAFLUSH, oldterm)
    if result:
        return ord(result)
    else:
        return 0



# *************************************************************************
#
# Copyright (c) 2021 Andrei Gramakov. All rights reserved.
#
# This file is licensed under the terms of the MIT license.
# For a copy, see: https://opensource.org/licenses/MIT
#
# site:    https://agramakov.me
# e-mail:  mail@agramakov.me
#
# *************************************************************************


class Register(int):
    def __new__(cls, address: int, *args, **kwargs):
        if address < 0:
            raise ValueError("positive types must not be less than zero")
        return super().__new__(cls, address)

    def __init__(self, address: int, access_type: str = "", comment="", *args, **kwargs) -> None:
        super().__init__()
        self.access_type = access_type
        self.comment = comment


class REGS:
    class MOVING_PLATFORM:
        CMD = Register(0x00, "R/W", "Command register. The device is reading commands from here")
        ARG = Register(0x01, "R/W", "Optional argument of the command")
        MODE = Register(0x02, "RO", "The Mode register")
        SPEED = Register(0x03, "RO", "Current speed mode (0-3)")
        ANGLE_X_S = Register(0x04, "RO", "Sing of the angle for X (0,1)")
        ANGLE_X = Register(0x05, "RO", "The value of the X angle (0-180)")
        ANGLE_Y_S = Register(0x05, "RO", "Sing of the angle for Y (0,1)")
        ANGLE_Y = Register(0x06, "RO", "The value of the Y angle (0-180)")
        ANGLE_Z_S = Register(0x07, "RO", "Sing of the angle for Z (0,1)")
        ANGLE_Z = Register(0x08, "RO", "The value of the Z angle (0-180)")

    class SENSOR_PLATFORM:
        CMD = Register(0x00, "R/W", "Command register. The device is reading commands from here")
        ARG = Register(0x01, "R/W", "Optional argument of the command")
        DIST_L = Register(0x02, "RO", "Value of the left distance sensor in cm")
        DIST_C = Register(0x03, "RO", "Value of the central distance sensor in cm")
        DIST_R = Register(0x04, "RO", "Value of the right distance sensor in cm")
        LIGHT_HI = Register(0x05, "RO", "Value of the light sensor, higher part. The less the brighter")
        LIGHT_LO = Register(0x06, "RO", "Value of the light sensor, lower part. The less the brighter")

    class FACE_MODULE:
        CMD = Register(0x00, "R/W", "Command register. The device is reading commands from here")
        ARG = Register(0x01, "R/W", "Optional argument of the command")



# *************************************************************************
#
# Copyright (c) 2021 Andrei Gramakov. All rights reserved.
#
# This file is licensed under the terms of the MIT license.
# For a copy, see: https://opensource.org/licenses/MIT
#
# site:    https://agramakov.me
# e-mail:  mail@agramakov.me
#
# *************************************************************************

from .low_level_i2c import i2c_write_byte_data, i2c_read_byte_from
from time import sleep


def cmd(addr, cmd, arg=0x0, wait_for_exec=False):
    if arg:
        i2c_write_byte_data(addr, 0x1, arg)  # write
    i2c_write_byte_data(addr, 0, cmd)  # write
    while (wait_for_exec and i2c_read_byte_from(addr, 0x0)):
        sleep(.1)
    sleep(.1)


def read(addr, reg):
    return i2c_read_byte_from(addr, reg)


def write(addr, reg, val):
    i2c_write_byte_data(addr, reg, val)  # write



# *************************************************************************
#
# Copyright (c) 2021 Andrei Gramakov. All rights reserved.
#
# This file is licensed under the terms of the MIT license.
# For a copy, see: https://opensource.org/licenses/MIT
#
# site:    https://agramakov.me
# e-mail:  mail@agramakov.me
#
# *************************************************************************

from smbus2 import SMBus
from threading import Lock

bus = SMBus(1)  # indicates /dev/i2c-1
i2c_mutex = Lock()


def i2c_read_byte(addr):
    global i2c_mutex
    with i2c_mutex:
        b = bus.read_byte(addr)
        print('Read byte %s from %s' % (hex(b), hex(addr)))
        return b


def i2c_write_byte(addr, val):
    global i2c_mutex
    with i2c_mutex:
        bus.write_byte(addr, val)
        print('Wrote byte %s to %s' % (hex(val), hex(addr)))


def i2c_write_byte_data(addr, reg, val):
    global i2c_mutex
    with i2c_mutex:
        bus.write_byte_data(addr, reg, val)
        print('Wrote byte %s from %s:%s' % (hex(val), hex(addr), hex(reg)))


def i2c_read_byte_from(addr, reg):
    global i2c_mutex
    with i2c_mutex:
        b = bus.read_byte_data(addr, reg)
        return b


def i2cdetect(bus_num):
    b = SMBus(bus_num)
    devs = []
    for a in range(0xff):
        try:
            b.read_byte(a)
            devs.append(hex(a))
        except IOError:
            pass
    return devs



# *************************************************************************
#
# Copyright (c) 2021 Andrei Gramakov. All rights reserved.
#
# This file is licensed under the terms of the MIT license.
# For a copy, see: https://opensource.org/licenses/MIT
#
# site:    https://agramakov.me
# e-mail:  mail@agramakov.me
#
# *************************************************************************

from . import low_level_i2c
from .funcs import cmd, write, read



# *************************************************************************
#
# Copyright (c) 2021 Andrei Gramakov. All rights reserved.
#
# This file is licensed under the terms of the MIT license.
# For a copy, see: https://opensource.org/licenses/MIT
#
# site:    https://agramakov.me
# e-mail:  mail@agramakov.me
#
# *************************************************************************

import pprint


def strf(in_str):
    pp = pprint.PrettyPrinter(indent=4, width=60, depth=30)
    return pp.pformat(in_str)


def list2strf(in_list, cell_size, in_hex=False):
    s = "[ "
    cell = ""
    for i in in_list:
        if in_hex:
            cell = hex(i)[2:]
        else:
            cell = str(i)
        c_l = len(cell)
        if c_l > cell_size:
            cell = cell[:cell_size - 1] + "?"
        if c_l < cell_size:
            cell = " " * (cell_size - c_l) + cell
        s = s + cell + " "
    s = s + "]"
    return s



# *************************************************************************
#
# Copyright (c) 2021 Andrei Gramakov. All rights reserved.
#
# This file is licensed under the terms of the MIT license.
# For a copy, see: https://opensource.org/licenses/MIT
#
# site:    https://agramakov.me
# e-mail:  mail@agramakov.me
#
# *************************************************************************


def get_round(in_list, num):
    new_l = []
    for i in in_list:
        new_l.append(round(i, num))
    return new_l


def get_max_deviation(in_list):
    lmax = float(max(in_list))
    lmin = float(min(in_list))
    delta = lmax - lmin
    if not delta:
        return 0
    return delta / lmax



# *************************************************************************
#
# Copyright (c) 2021 Andrei Gramakov. All rights reserved.
#
# This file is licensed under the terms of the MIT license.
# For a copy, see: https://opensource.org/licenses/MIT
#
# site:    https://agramakov.me
# e-mail:  mail@agramakov.me
#
# *************************************************************************


class CMD:
    class MOVE:
        FORWARD = 119
        BACKWARD = 115
        LEFT = 97
        RIGHT = 100
        SHIVER = 0x71
        STOP_KB = 0x20
        STOP = 0xA0
        SPEED0 = 0x30
        SPEED1 = 0x31
        SPEED2 = 0x32
        SPEED3 = 0x33
        MPU_CALIBRATE = 99
        TEST = 0x74

    class FACE:
        CALM = 0x30
        BLINK = 0x31
        ANGRY = 0x32
        HAPPY = 0x33
        SAD = 0x34



# *************************************************************************
#
# Copyright (c) 2021 Andrei Gramakov. All rights reserved.
#
# This file is licensed under the terms of the MIT license.
# For a copy, see: https://opensource.org/licenses/MIT
#
# site:    https://agramakov.me
# e-mail:  mail@agramakov.me
#
# *************************************************************************

from .addr import ADDR
from .cmd import CMD
from .regs import REGS
